#include <iostream>
#include <coroutine>

// Die Coroutinen-Klasse
struct MyCoroutine
{
    struct promise_type;
    using handle = std::coroutine_handle<promise_type>;
    // promise_type Klasse
    struct promise_type
    {
        // Aufruf unmittelbar nach Start der Coroutine
        auto initial_suspend()
        {
            std::cout << "initial_suspend()\n";
            return std::suspend_always{};
        }
        // Aufruf nach Beenden der Coroutine
        auto final_suspend() noexcept
        {
            std::cout << "final_suspend()\n";
            return std::suspend_never{};
        }
        // Aufruf beim Ausloesen einer nicht behandelten
        // Ausnahme
        void unhandled_exception()
        {
            std::cout << "unhandled_exception\n";
            std::terminate();
        }
        // Liefert das Coroutinen-Objekt zurueck
        auto get_return_object()
        {
            std::cout << "get_return_object()\n";
            return MyCoroutine{ handle::from_promise(*this) };
        }
        // Aufruf durch co_return
        void return_void()
        {
            std::cout << "return_void()\n";
        }
    };
    // Setzt die Ausfuehrung der Coroutine fort (coro.resume()),
    // wenn das Coroutinen-Handle gueltig ist (ungleich 0)
    bool resume()
    {
        return coro ? (coro.resume(), !coro.done()) : false;
    }
private:
    // Coroutinen-Handle
    handle coro;
    // ctor, speichert das uebergebene Coroutinen-Handle ab
    MyCoroutine(handle h) : coro(h)
    {
        std::cout << "ctor MyCoroutine\n";
    }
};

// Die Coroutine
// Rueckgabetyp ist immer die Coroutinen-Klasse
// Ausser der Ausgabe von Text passiert hier nichts
MyCoroutine CoTask()
{
    std::cout << "+++Start CoTask()\n";
    std::cout << "---Ende CoTask()\n";
    // Coroutine beenden, ohne einen Rueckgabewert
    // co_return definiert die Funktion fuer den Compiler
    // als eine Coroutine!!
    co_return;
}

int main()
{
    std::cout << "Erstelle Coroutine\n";
    // Coroutinen-Objekt erstellen
    // coro1 ist das Ergebnis des Aufrufs von get_return_object()
    auto coro1 = CoTask();
    // Coroutine fortsetzen, da in initial_suspend()
    // die Coroutine unterbrochen wurde
    std::cout << "resume()\n";
    coro1.resume();
    std::cout << "Ende main()\n";
}
