#include <iostream>
#include <format>
#include <coroutine>
#include <fstream>
#include <string>

// Die Coroutinen-Klasse
struct MyCoroutine
{
    struct promise_type;
    using handle = std::coroutine_handle<promise_type>;
    // promise_type Klasse
    struct promise_type
    {
        // Aufruf unmittelbar nach Start der Coroutine
        auto initial_suspend()
        {
            return std::suspend_always{};
        }
        // Aufruf nach Beenden der Coroutine
        auto final_suspend() noexcept
        {
            return std::suspend_always{};
        }
        // Aufruf beim Ausloesen einer nicht behandelten
        // Ausnahme
        void unhandled_exception()
        {
            std::terminate();
        }
        // Liefert das Coroutinen-Objekt zurueck
        auto get_return_object()
        {
            return MyCoroutine{ handle::from_promise(*this) };
        }
        // Aufruf durch co_return
        void return_void()
        {}
    };
    // Setzt die Ausfuehrung der Coroutine fort (coro.resume()),
    // wenn das Coroutinen-Handle gueltig ist (ungleich 0)
    bool resume()
    {
        return coro ? (coro.resume(), !coro.done()) : false;
    }
private:
    // Coroutinen-Handle
    handle coro;
    // ctor, speichert das uebergebene Coroutinen-Handle ab
    MyCoroutine(handle h) : coro(h)
    {
    }
};

// Die Coroutine
MyCoroutine CoTask()
{
    // Eingabedatei oeffnen und eventl. Fehler abfangen
    std::ifstream infile("../../04_Data/airports.csv");
    if (!infile)
    {
        std::cout << "Fehler beim Oeffnen der Datei!\n";
        co_return;
    }
    // Flughafen-Zaehler
    int counter = 0;
    do
    {
        std::string line;
        // Zeile aus Datei einlesen
        std::getline(infile, line);
        // Wenn Dateiende erreicht, Rest ueberspringen
        if (infile.eof())
            continue;
        // Wenn erster Buchstabe ein E ist
        // Flughafen-Namen ausgeben und Zaehler inkr.
        if (line[0] == 'E')
        {
            std::cout << line << '\n';
            counter++;
        }
    } while (!infile.eof());
    // Und Datei wieder schliessen!!
    infile.close();
    std::cout << std::format("\nAnzahl Flughaefen mit Code EXXX :{}\n", counter);
    // Coroutine beenden, ohne einen Rueckgabewert
    co_return;
}

int main()
{
    // Coroutinen-Objekt erstellen
    // coro1 ist das Ergebnis des Aufrufs von get_return_object()
    auto coro1 = CoTask();
    // Coroutine fortsetzen, da in initial_suspend()
    // die Coroutine unterbrochen wurde
    coro1.resume();
}
