#include <iostream>
#include <format>
#include <coroutine>
#include <fstream>
#include <string>

// Die Coroutinen-Klasse
struct MyCoroutine
{
    struct promise_type;
    using handle = std::coroutine_handle<promise_type>;
    // promise_type Klasse
    struct promise_type
    {
        // Letzter aus der Coroutine zurueckgegebener Wert
        int saved_value;
        // Aufruf unmittelbar nach Start der Coroutine
        auto initial_suspend()
        {
            return std::suspend_always{};
        }
        // Aufruf nach Beenden der Coroutine
        auto final_suspend() noexcept
        {
            return std::suspend_always{};
        }
        // Aufruf beim Ausloesen einer nicht behandelten
        // Ausnahme
        void unhandled_exception()
        {
            std::terminate();
        }
        // Liefert das Coroutinen-Objekt zurueck
        auto get_return_object()
        {
            return MyCoroutine{ handle::from_promise(*this) };
        }
        // Aufruf durch co_return
        void return_value(int value)
        {
            saved_value = value;
        }
    };
    // Setzt die Ausfuehrung der Coroutine fort (coro.resume()),
    // wenn das Coroutinen-Handle gueltig ist (ungleich 0)
    bool resume()
    {
        return coro ? (coro.resume(), !coro.done()) : false;
    }
    int getResult()
    {
        return coro.promise().saved_value;
    }
private:
    // Coroutinen-Handle
    handle coro;
    // ctor, speichert das uebergebene Coroutinen-Handle ab
    MyCoroutine(handle h) : coro(h)
    {
    }
};

// Die Coroutine
// search enthaelt den Anfangsbuchstaben der zu
// suchenden Flughafen-Codes
MyCoroutine CoTask(char search)
{
    // Eingabedatei oeffnen und eventl. Fehler abfangen
    std::ifstream infile("../../04_Data/airports.csv");
    if (!infile)
    {
        std::cout << "Fehler beim Oeffnen der Datei!\n";
        co_return 0;
    }
    // Flughafen-Zaehler
    int counter = 0;
    do
    {
        std::string line;
        // Zeile aus Datei einlesen
        std::getline(infile, line);
        // Wenn Dateiende erreicht, Rest ueberspringen
        if (infile.eof())
            continue;
        // Wenn erster Buchstabe gleich 'search' ist
        // Flughafen-Namen ausgeben und Zaehler inkr.
        if (line[0] == search)
        {
            std::cout << line << '\n';
            counter++;
        }
    } while (!infile.eof());
    // Und Datei wieder schliessen!!
    infile.close();
    // Coroutine beenden, gefundene Anzahl zurueckgeben
    co_return counter;
}

int main()
{
    char search = 'K';  // Zu suchende Flughafen-Codes
    // Coroutinen-Objekt erstellen
    // coro1 ist das Ergebnis des Aufrufs von get_return_object()
    auto coro1 = CoTask(search);
    // Coroutine fortsetzen, da in initial_suspend()
    // die Coroutine unterbrochen wurde
    coro1.resume();
    // Ergebnis der Couroutine ausgeben
    std::cout << std::format("{} Flughaefen haben den Code {}xxx\n",
                             coro1.getResult(), search);
}
