#include <iostream>
#include <format>
#include <coroutine>
#include <fstream>
#include <string>

// Die Coroutinen-Klasse
struct MyCoroutine
{
    struct promise_type;
    using handle = std::coroutine_handle<promise_type>;
    // promise_type Klasse
    struct promise_type
    {
        // Mittels co_return zurueckgegebener Wert
        int saved_value;
        // Mittels co_yield zugegebener Flughafen
        std::string airport;
        // Aufruf unmittelbar nach Start der Coroutine
        auto initial_suspend()
        {
            saved_value = 0;
            return std::suspend_always{};
        }
        // Aufruf nach Beenden der Coroutine
        auto final_suspend() noexcept
        {
            return std::suspend_always{};
        }
        // Aufruf beim Ausloesen einer nicht behandelten
        // Ausnahme
        void unhandled_exception()
        {
            std::terminate();
        }
        // Liefert das Coroutinen-Objekt zurueck
        auto get_return_object()
        {
            return MyCoroutine{ handle::from_promise(*this) };
        }
        // Aufruf durch co_return
        void return_value(int value)
        {
            saved_value = value;
        }
        // Aufruf durch co_yield
        auto yield_value(const std::string& ap)
        {
            airport = ap;
            return std::suspend_always{};
        }
    };
    // Setzt die Ausfuehrung der Coroutine fort (coro.resume()),
    // wenn das Coroutinen-Handle gueltig ist (ungleich 0)
    bool resume()
    {
        return coro ? (coro.resume(), !coro.done()) : false;
    }
    int getResult()
    {
        return coro.promise().saved_value;
    }
    const std::string_view getAirport()
    {
        return coro.promise().airport;
    }
private:
    // Coroutinen-Handle
    handle coro;
    // ctor, speichert das uebergebene Coroutinen-Handle ab
    MyCoroutine(handle h) : coro(h)
    {
    }
};

// Die Coroutine
// search enthaelt den Anfangsbuchstaben der zu
// suchenden Flughafen-Codes
MyCoroutine CoTask(char search)
{
    // Eingabedatei oeffnen und eventl. Fehler abfangen
    std::ifstream infile("../../04_Data/airports.csv");
    if (!infile)
    {
        std::cout << "Fehler beim Oeffnen der Datei!\n";
        co_return 0;
    }
    // Flughafen-Zaehler
    int counter = 0;
    do
    {
        std::string line;
        // Zeile aus Datei einlesen
        std::getline(infile, line);
        // Wenn Dateiende erreicht, Rest ueberspringen
        if (infile.eof())
            continue;
        // Wenn erster Buchstabe gleich 'search' ist
        // Zaehler inkr. und Flughafen zurueckgeben
        if (line[0] == search)
        {
            counter++;
            co_yield line;
        }
    } while (!infile.eof());
    // Und Datei wieder schliessen!!
    infile.close();
    // Coroutine beenden, gefundene Anzahl zurueckgeben
    co_return counter;
}

int main()
{
    // Zu suchende Flughafen-Codes
    char search;
    std::cout << "Zu suchende Flughafen-Codes? ";
    std::cin >> search;
    // Coroutinen-Objekt erstellen
    // coro1 ist das Ergebnis des Aufrufs von get_return_object()
    auto coro1 = CoTask(search);
    // Abfrageintervall ob weiter gesucht werden soll
    const int noPrint = 5;
    int apCounter = 0;
    // Coroutine solange durchlaufen, bis resume() false
    // zurueckliefert, d.h die Coroutine alles gefunden hat
    while (coro1.resume())
    {
        std::cout << coro1.getAirport() << '\n';
        if (apCounter++ == noPrint)
        {
            char again;
            std::cout << "Weiter suchen (J/N)? ";
            std::cin >> again;
            if ((again != 'J') && (again != 'j'))
                break;
            else
                apCounter = 0;
        }
    }
    // Ergebnis der Couroutine ausgeben
    apCounter = coro1.getResult();
    if (apCounter != 0)
        std::cout << std::format("{} Flughaefen haben den Code {}xxx\n",
            apCounter, search);
    else
        std::cout << "Kein Flughafen gefunden oder Suche abgebrochen.\n";
}
