module;
#include <iostream>
#include <string>
#include <format>

// Makro fr Debug-Ausgaben
//#define TRACE_METH
#ifdef TRACE_METH
#define TRACE(T_)   std::cout <<  "Obj-Nr:" << num << ' ' \
    << T_ << ',' << text << std::endl
#else
#define TRACE(T_)
#endif

// Modul exportieren
export module Demo;

// Definition Demo-Klasse
export class Demo
{
    std::string text;       // Nutzdatum
    static int count;       // statische Objektzaehler
    int num;                // Objektnummer
public:
    // Standard-ctor
    Demo()
    {
        num = ++count;
        TRACE("Standard-ctor");
    }
    // ctor mit Parameter
    Demo(const char* v): text(v)
    {
        num = ++count;
        TRACE("Parameter-ctor");
    }
    // Kopier-ctor
    Demo(const Demo& obj): text(obj.text)
    {
        num = ++count;
        TRACE("copy-ctor");
    }
    // move-ctor
    Demo(const Demo&& obj)
    {
        text = obj.text;
        num = obj.num;
        TRACE("move-ctor");
    }
    // dtor
    ~Demo()
    {
        TRACE("dtor");
    }
    // Zuweisungsoperator
    // Objektnummer bleibt erhalten, nur text uebernehmen
    Demo& operator = (const Demo& obj)
    {
        text = obj.text;
        TRACE("operator =");
        return *this;
    }
    // move-Zuweisungsoperator
    // Objektnummer wird uebernommen
    Demo& operator = (const Demo&& obj)
    {
        text = obj.text;
        num = obj.num;
        TRACE("move operator =");
        return *this;
    }
    const std::string& GetText() const
    {
        return text;
    }
    // Ausgabe der Eigenschaften ueber Operator <<
    friend std::ostream& operator << (std::ostream& os, const Demo& obj);
    // friend-Template wegen Zugriff auf private-Eigenschaften
    friend struct std::formatter<Demo>;

};
// Statischen Objektzaehler definieren/initialisieren
decltype(Demo::count)  Demo::count = 0;

export std::ostream& operator << (std::ostream& os, const Demo& obj)
{
    os << obj.GetText();
    return os;
}
// Template-Spezialisierung fuer Ausgabe mittels format()
template<>
struct std::formatter<Demo>
{
    constexpr auto parse(auto& ctx)
    {
        return ctx.begin();
    }
    auto format(const Demo& obj, std::format_context& ctx)
    {
        return std::format_to(ctx.out(), "{}", obj.GetText());
    }
};
