// Dateien einbinden
#include <iostream>
#include <cmath>
#include <numbers>
#include <format>

// Gravitationskonstante
constexpr auto GRAVITATION = 9.81;
// Konstante für Umrechnung Grad->Rad
constexpr auto PI2R = 2.0*std::numbers::pi;

// Funktionsdeklarationen
double Width(unsigned long velocity, short angle);
double Height(unsigned long velocity, short angle);

// main() Funktion
int main()
{
   // Variablen für Abwurfgeschw. und -winkel definieren
   unsigned long actVelocity;

   // Wurfdaten bei konst. Abwurfwinkel 45 Grad berechnen
   auto angle = 45;
   std::cout << std::format("Schraeger Wurf mit konst. Winkel von {} Grad\n", angle);
   // Abwurfgeschwindigkeit variieren
   for (actVelocity=10; actVelocity<=20; actVelocity+=2)
   {
       std::cout << std::format("Geschw. (m/s): {}, Weite (m): {:4.2f}, Hoehe (m): {:4.2f}\n",
           actVelocity, Width(actVelocity, angle), Height(actVelocity, angle));
   }

   // Wurfdaten bei konst. Abwurfgeschw. 28 m/s berechnen
   actVelocity = 28;
   std::cout << std::format("\nSchraeger Wurf mit konst. Abwurfgeschw. {} m/s\n", actVelocity);
   // Wurfwinkel variieren
   for (angle=30; angle<=60; angle+=5)
   {
       std::cout << std::format("Winkel (Grad): {}, Weite (m):{:4.2f}, Hoehe (m): {:4.2f}\n",
           angle, Width(actVelocity, angle), Height(actVelocity, angle));
   }
}

// Berechnung der Weite eines schrägen Wurfs
// velocity : Abwurfgeschwindigkeit (m/s)
// angle : Abwurfwinkel (Grad)
// Rückgabe: Weite (m)
double Width(unsigned long velocity, short angle)
{
   // Grad in Rad konvertieren für Sinus-Funktion
   auto dfRad = angle/360.0*PI2R;
   // Weite berechnen und zurückliefern
   decltype(Width(1,1)) dfWeite = velocity*velocity*sin(dfRad*2.0)/GRAVITATION;
   return dfWeite;
}

// Berechnung der max. Höhe eines schrägen Wurfs
// velocity : Abwurfgeschwindigkeit (m/s)
// angle : Abwurfwinkel (Grad)
// Rückgabe: max. Höhe (m)
double Height(unsigned long velocity, short angle)
{
   // Grad in Rad konvertieren für Sinus-Funktion
   auto dfRad = angle/360.*PI2R;
   // Sinus nur einmal berechnen, wird 2-mal in Formel benötigt
   auto dfSin = sin(dfRad);
   // max. Höhe berechnen und zurückliefern
   decltype(Height(1,1)) dfHoehe = velocity*velocity * dfSin*dfSin/(2.0*GRAVITATION);
   return dfHoehe;
}
