#include <iostream>
#include <format>
#include <fstream>
#include <vector>
#include <algorithm>

// Dateiname
constexpr const char* FILENAME = "../../04_Data/verbrauch.txt";
// struct zum Abspeiohern der Tankdaten
struct data
{
    unsigned int km;        // km-Stand
    unsigned int ltr;       // getankte Liter
    float        verbr;     // ltr/100km
};

// Tankdaten
std::vector<data> verbrauch;

int main()
{
    // Datei mit Tankdaten oeffnen
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        std::cout << std::format("Datei {} nicht gefunden!\n",FILENAME);
        exit(1);

    }
    // Tankdaten einlesen
    do
    {
        decltype(data::km)  km;        // akt. km-Stand
        decltype(data::ltr) tank;      // getankte ltr.
        infile >> km >> tank;
        if (infile.eof())
            continue;
        // Tankdaten im Vektor ablegen, Verbrauch mit 0.0 belegen
        verbrauch.emplace_back(km,tank,0.0f);
    } while (!infile.eof());
    // Datei schliesen
    infile.close();

    // vorheriger km-Stand
    unsigned int km_alt = 0;
    // Tankdaten durchlaufen um Verbrauch zu berechnen
    // Verbrauch wird in einer Lambda-Funktion berechnet
    std::ranges::for_each(verbrauch,
              // Lambda-Funktion, erhaelt vorherigen km-Stand
              // per Referenz und um Parameter akt. Tankdaten
              // Der 1. Eintrag enthaelt nur km-Stand bei Uebergabe
              // Fuer nachfolgende Eintraege Verbrauch berechnen
              // und im Vektor ablegen. Anschließend den akt.
              // km-Stand merken
             [&km_alt] (data& eintrag)
                  {
                      if (km_alt == 0)
                      {
                          km_alt = eintrag.km;
                      }
                      else
                      {
                          eintrag.verbr = static_cast<float>(eintrag.ltr) /
                              static_cast<float>(eintrag.km-km_alt)*100.0f;
                          km_alt = eintrag.km;
                      }
                  }
             );

    // Ausgabe der Tankdaten inkl. Verbrauch (auf 2 Dezimalstellen)
    std::cout << "Tankfuellung:\n";
    for (auto [km, tank, verbr] : verbrauch)
        std::cout << std::format("km: {}, getankt: {}, Verbrauch: {:.2f}\n",
            km, tank, verbr);

    // Minimalen und maximalen Verbrauch
    // Vergleich mit Default-Predicate less{} erst ab 2. Element im Vektor
    // Ausgewertet wird Member verbr in der Klasse data (Projektion!)
    auto [minv, maxv] = std::ranges::minmax_element(verbrauch.begin() + 1, verbrauch.end(),
        {}, &data::verbr);
    std::cout << std::format("\nMin. Verbrauch: {:.02f}, max. Verbrauch: {:.02f}\n",
                             minv->verbr, maxv->verbr);
}
