module;
#include <iostream>
#include <format>
#include <string>
#include <string_view>

export module TestCase;

// Klassendefinition
export class CTestCase
{
public:
    // enum der Prioritäten
    enum class priority {low, medium, high, critical};
    // enum des Prüffallstatus
    enum class state {defined, failed, ok};
private:
    std::string description;    // Beschreibung
    unsigned int tcNumber;      // Nummer
    priority prio;              // Priorität
    state tcState;              // Status
    inline static unsigned int number = 1; // Prüffallindex
  public:
    // ctor
    CTestCase(std::string_view desc, priority p);
    // Default copy-ctor da keine dyn. Eigenschaften vorhanden
    CTestCase(const CTestCase&) = default;
    // Default dtor
    ~CTestCase() = default;
    // Zuweisung der Priorität
    CTestCase& operator = (priority p);
    // Zuweisung des Status
    CTestCase& operator = (state s);
    // Ausgabe des Prüffalls
    void PrintIt () const;
};
// Definition der Memberfunktion
// ctor
CTestCase::CTestCase(std::string_view desc, priority p) :
    description(desc)
{
    tcNumber = number; // Nummber ablegen
    number++; // Pruffallindex erhöhen
    tcState = state::defined; // Status setzen
    prio = p; // Priorität ablegen
 }
// Zuweisung des Status
inline CTestCase& CTestCase::operator = (state s)
{
    tcState = s;
    return *this;
}
// Zuweisung der Priorität
inline CTestCase& CTestCase::operator = (priority p)
{
    prio = p;
    return *this;
}
// Ausgabe des Prüffalls
void CTestCase::PrintIt () const
{
    std::cout << std::format("Prueffall: {}\nNummer: {}, Prio: {}, Status: ",
        description, tcNumber, static_cast<int>(prio));
    switch (tcState)
    {
        using enum state;
    case defined:
        std::cout << "definiert\n\n";
        break;
    case failed:
        std::cout << "fehlerhaft\n\n";
        break;
    case ok:
        std::cout << "erfolgreich\n\n";
        break;
    default:
        std::cout << "unbekannt\n\n";
    }
}
