module;
#include <cstring>

export module CString;

// Klassendefinition
export class CString
{
    char* pString; // String
public:
    // ctor
    CString();
    CString(const char* const psT);
    CString(const CString& source);
    // dtor
    ~CString();
    // Strings zuweisen
    CString& operator = (const char* const pS);
    CString& operator = (const CString& source);
    // Strings addieren
    void AddString(const char* const pS);
    void AddString(const CString& add);
    // String ausgeben
    const char* GetString() const;
};

// Definition der Memberfunktionen
// 1. Konstruktor, delegation ctor!
CString::CString() : CString("")
{
}
// 2. Konstruktor (target ctor)
CString::CString(const char* const pS)
{

    // Nun neuen Speicherplatz für String anfordern
    // Die abschliessende binäre 0 dabei nicht vergessen!
    auto len = strlen(pS) + 1;
    pString = new char[len];
    // String umkopieren
    strcpy_s(pString, len, pS);
}
// 3. Konstruktor (Kopierkonstruktor,delegation ctor!)
CString::CString(const CString& source) : CString(source.GetString())
{
}
// Destruktor
CString::~CString()
{
    // Speicher für char-Feld freigeben!
    delete[] pString;
}
// Weist einem String Objekt einen char-String zu
CString& CString::operator = (const char* const pS)
{
    // Alten String löschen
    delete[] pString;
    // Neuen String anlegen
    auto len = strlen(pS) + 1;
    pString = new char[len];
    strcpy_s(pString, len, pS);
    return *this;
}
// Weist einem String Objekt ein anderes String Objekt zu
CString& CString::operator = (const CString& source)
{
    // Zuweisung auf sich selbst abprüfen
    if (&source == this)
        return *this;
    else
        // Zuweisung ausfuehren
        return operator = (source.pString);
}
// Fügt den über den char-Zeiger übergebenen String hinzu
void CString::AddString(const char* const pS)
{
    // Zuerst Speicher für neuen String reservieren
    auto bufSize = strlen(pString) + strlen(pS) + 1;
    auto pTemp = new char[bufSize];
    // Dann den ursprünglichen String umkopieren
    strcpy_s(pTemp, bufSize, pString);
    // und nun den neuen String hinzufügen
    strcat_s(pTemp, bufSize, pS);
    // Jetzt erst den ursprünglichen Speicher freigeben
    delete[] pString;
    // und Zeiger auf neuen Speicherbereich setzen
    pString = pTemp;
}
// Fügt den über die String-Referenz übergebenen String hinzu
void CString::AddString(const CString& add)
{
    // Gleicher Ablauf wie vorherige Memberfunktion
    AddString(add.pString);
}
// Ausgabe des Strings
// Ausgabe des Strings
inline const char* CString::GetString() const
{
    return pString;
}
