module;
#include <cstring>
#include <compare>

export module CString;

// Klassendefinition
export class CString
{
    char* pString; // String
public:
    // ctor
    CString();
    CString(const char* const psT);
    CString(const CString& source);
    // dtor
    ~CString();
    // Strings zuweisen
    CString& operator = (const char* const pS);
    CString& operator = (const CString& source);
    CString& operator = (CString&& source);
    // Strings addieren
    CString operator + (const char* const pS) const;
    CString operator + (const CString& op2) const;
    // Strings vergleichen
    auto operator <=> (const CString& rhs) const;
    bool operator == (const CString& rhs) const;
    // String ausgeben
    const char* GetString() const;
};

// Definition der Memberfunktionen
// 1. Konstruktor, delegation ctor!
CString::CString() : CString("")
{
}
// 2. Konstruktor (target ctor)
CString::CString(const char* const pS)
{

    // Nun neuen Speicherplatz für String anfordern
    // Die abschliessende binäre 0 dabei nicht vergessen!
    auto len = strlen(pS) + 1;
    pString = new char[len];
    // String umkopieren
    strcpy_s(pString, len, pS);
}
// 3. Konstruktor (Kopierkonstruktor,delegation ctor!)
CString::CString(const CString& source) : CString(source.GetString())
{
}
// Destruktor
CString::~CString()
{
    // Speicher für char-Feld freigeben!
    delete[] pString;
}
// Weist einem String Objekt einen char-String zu
CString& CString::operator = (const char* const pS)
{
    // Alten String löschen
    delete[] pString;
    // Neuen String anlegen
    auto len = strlen(pS) + 1;
    pString = new char[len];
    strcpy_s(pString, len, pS);
    return *this;
}
// Weist einem String Objekt ein anderes String Objekt zu
CString& CString::operator = (const CString& source)
{
    // Zuweisung auf sich selbst abprüfen
    if (&source == this)
        return *this;
    else
        // Zuweisung ausfuehren
        return operator = (source.pString);
}
// Move-Operator =
// Wird aufgerufen, wenn ein temporäres CString-
// Objekt zugewiesen wird
CString& CString::operator = (CString&& source)
{
    if (this == &source)
        return *this;
    // Speicher-Ressource uebertragen
    pString = source.pString;
    // Quell-Objekt in gueltigen Zustand bringen
    source.pString = new char[1];
    source.pString[0] = 0;
    return *this;
}
// Fügt den über den char-Zeiger übergebenen String hinzu
CString CString::operator + (const char* const pS) const
{
    // Temp. Objekt
    CString temp;

    // Zuerst den Speicher des temp. Objekts freigeben!
    delete[] temp.pString;

    // Nun Speicher für neuen String reservieren
    auto len = strlen(pString) + strlen(pS) + 1;
    temp.pString = new char[len];
    // Zuerst den ursprünglichen String umkopieren
    strcpy_s(temp.pString, len, pString);
    // dann den String des 2. Operanden hinzufügen
    strcat_s(temp.pString, len, pS);
    return temp;
}
// Fügt den über die String-Referenz übergebenen String hinzu
CString CString::operator + (const CString& op2) const
{
    return operator + (op2.pString);
}
auto CString::operator <=> (const CString& rhs) const
{
    return *pString <=> *(rhs.pString);
}
bool CString::operator == (const CString& rhs) const
{
    return strcmp(pString, rhs.pString) == 0;
}
// Ausgabe des Strings
// Ausgabe des Strings
inline const char* CString::GetString() const
{
    return pString;
}
