module;
#include <iostream>
#include <cstring>
#include <compare>

export module CString;

// Klassendefinition
export class CString
{
    char* pString; // String
public:
    // ctor
    CString();
    CString(const char* const psT);
    CString(const CString& source);
    // dtor
    ~CString();
    // Strings zuweisen
    CString& operator = (const char* const pS);
    CString& operator = (const CString& source);
    CString& operator = (CString&& source);
    // Strings addieren
    CString operator + (const char* const pS) const;
    CString operator + (const CString& op2) const;
    // Strings vergleichen
    auto operator <=> (const CString& rhs) const;
    bool operator == (const CString& rhs) const;
    // String ausgeben
    const char* GetString() const;
    // Überladener Indexoperator
    char& operator [] (const int index);
    // Überladener Ausgabe- und Eingabeoperator
    friend std::istream& operator >> (std::istream& is, CString& op2);
};

// Definition der Memberfunktionen
// 1. Konstruktor, delegation ctor!
CString::CString() : CString("")
{
}
// 2. Konstruktor (target ctor)
CString::CString(const char* const pS)
{

    // Nun neuen Speicherplatz für String anfordern
    // Die abschliessende binäre 0 dabei nicht vergessen!
    auto len = strlen(pS) + 1;
    pString = new char[len];
    // String umkopieren
    strcpy_s(pString, len, pS);
}
// 3. Konstruktor (Kopierkonstruktor,delegation ctor!)
CString::CString(const CString& source) : CString(source.GetString())
{
}
// Destruktor
CString::~CString()
{
    // Speicher für char-Feld freigeben!
    delete[] pString;
}
// Weist einem String Objekt einen char-String zu
CString& CString::operator = (const char* const pS)
{
    // Alten String löschen
    delete[] pString;
    // Neuen String anlegen
    auto len = strlen(pS) + 1;
    pString = new char[len];
    strcpy_s(pString, len, pS);
    return *this;
}
// Weist einem String Objekt ein anderes String Objekt zu
CString& CString::operator = (const CString& source)
{
    // Zuweisung auf sich selbst abprüfen
    if (&source == this)
        return *this;
    else
        // Zuweisung ausfuehren
        return operator = (source.pString);
}
// Move-Operator =
// Wird aufgerufen, wenn ein temporäres CString-
// Objekt zugewiesen wird
CString& CString::operator = (CString&& source)
{
    if (this == &source)
        return *this;
    // Speicher-Ressource uebertragen
    pString = source.pString;
    // Quell-Objekt in gueltigen Zustand bringen
    source.pString = new char[1];
    source.pString[0] = 0;
    return *this;
}
// Fügt den über den char-Zeiger übergebenen String hinzu
CString CString::operator + (const char* const pS) const
{
    // Temp. Objekt
    CString temp;

    // Zuerst den Speicher des temp. Objekts freigeben!
    delete[] temp.pString;

    // Nun Speicher für neuen String reservieren
    auto len = strlen(pString) + strlen(pS) + 1;
    temp.pString = new char[len];
    // Zuerst den ursprünglichen String umkopieren
    strcpy_s(temp.pString, len, pString);
    // dann den String des 2. Operanden hinzufügen
    strcat_s(temp.pString, len, pS);
    return temp;
}
// Fügt den über die String-Referenz übergebenen String hinzu
CString CString::operator + (const CString& op2) const
{
    return operator + (op2.pString);
}
auto CString::operator <=> (const CString& rhs) const
{
    return *pString <=> *(rhs.pString);
}
bool CString::operator == (const CString& rhs) const
{
    return strcmp(pString, rhs.pString) == 0;
}

// Ausgabe des Strings
// Ausgabe des Strings
inline const char* CString::GetString() const
{
    return pString;
}
// Der Operator[] soll den Zugriff auf ein bestimmtes Zeichen im Text
// erlauben. Ist der Index grösser als die Stringlänge wird eine
// Referenz auf die abschliessende binäre 0 zurückgegeben. Damit kann
// im Programm mit der Anweisung
// if (CStringObj[nIndex] == 0)
// abgefragt werden, ob über den String hinaus zugegriffen wird!
// Als Returnwert muss hier eine Referenz auf das entsprechende Zeichen
// angegeben werden und nicht das Zeichen selbst da sonst der Indexoperator
// nicht links vom Gleichheitszeichen stehen darf!
char& CString::operator [] (const int index)
{
    // Falls über den String hinaus zugegriffen wird
    if (index >= static_cast<int>(strlen(pString)) || (index < 0))
        // Referenz auf die abschliessende binäre 0 zurückgeben
        return pString[strlen(pString)];
    // Referenz auf das gewünschte Zeichen liefern
    return pString[index];
}
// Überladene Operatorfunktion zum Einlesen eines String Objekts
export std::istream& operator >> (std::istream& is, CString& op2)
{

    // Temp. Puffer für die Eingabe
    constexpr int BSIZE = 80;
    char acBuffer[BSIZE];
    // Eine Zeile einlesen
    is.getline(acBuffer, sizeof(acBuffer));
    // alten String freigeben
    delete[] op2.pString;
    // Platz für neuen String anfordern
    auto len = strlen(acBuffer) + 1;
    op2.pString = new char[len];
    // String jetzt umkopieren
    strcpy_s(op2.pString, len, acBuffer);
    // Referenz auf Stream zurückgeben
    return is;
}
