#include <iostream>
#include <fstream>
#include <format>
#include <string>
#include <charconv>
#include <algorithm>
#include <ranges>

// Datei mit den Aktienkursen
constexpr auto FILENAME = "../../04_Data/aktien.csv";
constexpr auto SIZE = 12;

// Feld zum Ablegen der Aktienkurse
struct Aktie
{
    std::string name;
    float       minVal;
    float       maxVal;
    std::string datum;
} aktien[SIZE];

int main()
{
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        std::cout << std::format("Datei {} wurde nicht gefunden!\n", FILENAME);
        exit(1);
    }
    // Daten einlesen
    int index = 0;
    do
    {
        // Hilfs-String, wegen Programmabsturz mit gcc
        // wenn getline() direkt in Strukturvariable einliest
        std::string line;
        // Name einlesen
        // Wenn EOF erreicht, Rest ueberspringen
        std::getline(infile, line, ';');
        if (infile.eof())
            continue;
         aktien[index].name=line;
        // Niedrigsten Kurs einlesen und Semikolon ueberspringen
        infile >> aktien[index].minVal;
        infile.ignore(1);
        // Hoechsten Kurs einlesen und Semikolon ueberspringen
        infile >> aktien[index].maxVal;
        infile.ignore(1);
        // Datum lesen (Rest der Zeile)
        getline(infile, aktien[index].datum);
        // Und Index erhoehen
        index++;
    } while (!infile.eof());

    // Aktie ausgeben
    auto PrintAktie = [](const Aktie& obj)
    {
        std::cout << std::format("AG: {}, min/max:  {:.2f}/{:.2f}\n",
            obj.name, obj.minVal, obj.maxVal, obj.datum);
    };

    // Alle Aktien ausgeben
    std::cout << "Aktienkurse in EUR:\n";
    std::ranges::for_each(aktien, PrintAktie);
    // Alle Kurswerte, deren AG mit dem Buchstaben D beginnen,
    // in Dollar umrechnen (1 EUR = 1.2 $) und ausgeben
    std::cout << "\nAktienkurse der AGs D* in Dollar:\n";
    auto afilter = [](const Aktie& obj) { return obj.name[0] == 'D'; };
    auto atrans = [](Aktie& obj) { obj.maxVal *= 1.2f; obj.minVal *= 1.2f; return obj; };
    std::ranges::for_each(aktien | std::views::filter(afilter)
        | std::views::transform(atrans) , PrintAktie);

}
