#include <iostream>
#include <format>
#include <string>
#include <string_view>
#include <queue>

// Klasse für Flugdaten
class Flight
{
    std::string     number;         // Flugnummer
    unsigned char   arrivalHour;    // Ankunftszeit Stunde
    unsigned char   arrivalMinute;  // Ankunftszeit Minute
  public:
    // ctor, erhält Flugnummer und Ankunftszeit
    Flight(std::string_view num, unsigned char hour, unsigned char minute): number(num)
    {
        arrivalHour = hour;
        arrivalMinute = minute;
    }
    // Vergleichsoperatoren für priority_queue
    // 
    auto operator <=> (const Flight& rhs) const
    {
        // Vergleiche Stunde
        auto res = rhs.arrivalHour <=> arrivalHour;
        // Wenn Stunde gleich, vergleich Minuten
        if (res == 0)
            res = arrivalMinute <=> rhs.arrivalMinute;
        return res;
    }
    // Überladener Ausgabestream Operator
    friend std::ostream& operator << (std::ostream& os, const Flight& rhs);
};

// Überladener Operator für Ausgabestream
inline std::ostream& operator << (std::ostream& os, const Flight& rhs)
{
    os << std::format("Flug: {:>6}, Ankunft: {:02d}:{:02d}\n",
        rhs.number, rhs.arrivalHour, rhs.arrivalMinute);
    return os;
}

// main() Funktion
int main()
{
    // Prioritätsqueue definieren
    std::priority_queue<Flight> Arrivals;

    // Flüge in Queue ablegen: flugnr, std, min
    Arrivals.push(Flight("EW 333",17,38));
    Arrivals.push(Flight("X 111",10,59));
    Arrivals.push(Flight("LH 444",10,38));
    Arrivals.push(Flight("UA 222",9,0));

    // Alle Flüge nun ausgeben und nach der Ausgabe
    // auch wieder entfernen
    while (!Arrivals.empty())
    {
        std::cout << Arrivals.top();
        Arrivals.pop();
    }
}
