#include <iostream>
#include <format>
#include <fstream>
#include <set>
#include <string>
#include <cctype>
#include <algorithm>
#include <ranges>

// Name der zu untersuchenden Textdatei
const std::string FILENAME("../../04_Data/cont3.txt");

// Lambda-Funktion zum Umwandeln in Kleinschreibung
auto LowerCase = [](char& letter)
{
    letter = std::tolower(letter);
};
// Lambda-Funktion zum Pruefen auf Anfangsbuchstaben
auto OnlyE = [check='e'](std::string_view word)
{
    return word[0] == check;
};

// main() Funktion
int main()
{
    // Zu untersuchende Datei öffnen
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        std::cout << std::format("Fehler beim Oeffnen der Datei {}\n", FILENAME);
        exit(1);
    }

    // Set für Woerter aus Datei anlegen
    std::set<std::string> dictionary;

    // Komplette Datei einlesen und Woerter ablegen
    std::cout << "Woerter ins Woerterbuch eintragen:\n";
    do
    {
        // Wort aus Datei einlesen
        std::string word;
        infile >> word;
        if (infile.eof())
            continue;
        // Alles in Kleinbuchstaben wandeln
        std::ranges::for_each(word, LowerCase);
        // Interpunktion am Wortende entfernen
        if (auto pos = word.find_first_of(".,!?"); pos != std::string::npos)
            word.erase(word.end() - 1);
        // Im Set ablegen. Falls Wort schon vorhanden, Meldung ausgeben
        if (auto [pos, success] = dictionary.insert(word); !success)
            std::cout << std::format("Wort '{}' schon vorhanden!\n", *pos);
    } while (!infile.eof());

    // Alle Woerter aus Datei ausgeben
    // Alternativ waere auch for_each() moeglich
    std::cout << "\nIn der Datei enthaltene Worte:\n";
    for (const auto& word : dictionary)
        std::cout << std::format("{}, ", word);
    std::cout << '\n';

    // Nun alle Woerter mit einem bestimmten Buchstaben ausgeben
    // Es werden nur die Woerter ausgegeben, fuer der Filter-View
    // true zurueck gibt.
    std::cout << "\nAlle Woerter mit dem Buchstaben 'e':\n";
    std::ranges::for_each(dictionary | std::views::filter(OnlyE),
        [](std::string_view word) { std::cout << word << ','; });
    std::cout << "\n\nAnzahl der Woerter (ohne Duplikate): " << dictionary.size() << std::endl;
}
