#include <iostream>
#include <format>
#include <map>
#include <string>
#include <utility>

// Adressbuch
// Eine Adresse besteht aus einem Namen, einer Telefonnr.
// und einer EMail-Adresse. Doppelte Namen sind nicht
// zulaessig
 class AddressBook
{
    // Adressdaten, Telfon/Email ohne Name
    using entry = std::pair<std::string, std::string>;
    // Map für Adressbuch-Einträge, Schluessel ist Name
    // Die Nutzdaten sind im pair entry zusammengefasst
    std::map<std::string, entry> theBook;
public:
    // ctor
    AddressBook() = default;
    // dtor
    ~AddressBook() = default;

    // Neue Adresse aufnehmen
    void Insert(std::string_view name, std::string_view phone, std::string_view email)
    {
        // Map benoetigt zum Einfuegen ein entsprechendes pair
        std::pair<std::string, entry> toInsert(name, entry(phone, email));
        // pair mit den Daten einfügen
        auto success = theBook.insert(toInsert);
        // Da wir nur eine Map haben sind keine doppelten Eintraege erlaubt
        if (!success.second)
            std::cout << std::format("{} konnte nicht eingefuegt werden!\n", name);
    }
    // Gibt einzelne Adresse aus
    void PrintAddress(std::string_view name) const
    {
        // Suche nach Namen
        // Falls Name nicht im Adressbuch
        if (auto iter = theBook.find(name.data()); iter == theBook.end())
            std::cout << "Kein entsprechender Eintrag gefunden!\n";
        else
        {
            // Sonst Adressdaten ausgeben, Iterator zeigt auf gefunden Eintrag
            std::cout << std::format("Name: {}\nTel.: {}, EMail: {}\n",
                iter->first, iter->second.first, iter->second.second);
        }
    }
    // Aendert Adressbuch-Eintrag
    bool Update(std::string_view name, std::string_view phone, std::string_view email)
    {
        bool success = false;
        // Eintrag suchen und falls gefunden
        if (auto iter = theBook.find(name.data()); iter == theBook.end())
        {
            std::cout << std::format("{} nicht im Adressbuch vorhanden!\n", name);
        }
        else
        {
            // Telefon-Nummer und Email korrigieren
            iter->second.first = phone;
            iter->second.second = email;
            success = true;
        }
        return success;
    }
    // Adresse entfernen
    void Remove(std::string_view name)
    {
        // Eintrag suchen
        if (auto iter = theBook.find(name.data()); iter == theBook.end())
            std::cout << std::format("{} nicht im Adressbuch vorhanden!\n", name);
        else
            // Wenn gefunden, dann loeschen
            theBook.erase(iter);
    }
    // Ausgabe des gesamten Buches
    void PrintBook()
    {
        for (auto elem : theBook)
            std::cout << std::format("Name: {}\nTel.: {}, EMail: {}\n",
                elem.first, elem.second.first, elem.second.second);
    }
};


// main() Funktion
int main()
{
    // Adressbuch dynamisch anlegen
    AddressBook *pAddr = new AddressBook;

    // Einige Eintraege hinzufuegen
    pAddr->Insert("Emil Maier", "0111/4545", "emai@sonst.net");
    pAddr->Insert("Agathe Maier", "0111/4545", "agma@sonst.net");
    pAddr->Insert("Franzi Xelsbrot", "08888/1234", "frax@kostnix.de");
    pAddr->Insert("Gustav Gans", "09999/1111", "guga@lucky.com");

    // Komplettes Adressbuch ausgeben
    std::cout << "Inhalt Adressbuch:\n";
    pAddr->PrintBook();

    // Einen Eintrag abaendern
    std::cout << "\nAendere Telefon-Nummer von Gustav Gans:\n";
    auto res = pAddr->Update("Gustav Gans", "09999/7777", "guga@lucky.com");
    // Und geaenderten Eintrag ausgeben
    if (!res)
        std::cout << "Aenderung fehlgeschlagen!\n";
    else
    {
        std::cout << "Neuer Eintrag ist:\n";
        pAddr->PrintAddress("Gustav Gans");
    }

    // Eintrag entfernen
    std::cout << "\nEntferne Franzi Xelsbrot:\n";
    pAddr->Remove("Franzi Xelsbrot");

    // Nochmals komplettes Adressbuch ausgeben
    std::cout << "\nNeuer Inhalt Adressbuch:\n";
    pAddr->PrintBook();

    // Adressbuch loeschen
    delete pAddr;
}
