#include <iostream>
#include <format>
#include <fstream>
#include <string>
#include <charconv>

// Datei mit den Verbrauchsdaten
constexpr const char* const FILENAME = "../../04_Data/verbrauch.txt";

// Funktion zur Berechnung des Durchschnittsverbrauch
inline float Verbrauch(int km, int ltr)
{
    return static_cast<float>(ltr) / static_cast<float>(km) * 100.0f;
}

int main()
{
    // Eingabedatei oeffnen
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        std::cout << std::format("Fehler beim Oeffnen der Datei {}\n", FILENAME);
        exit(1);
    }

    std::string data;           // einzulesende Zeile aus Datei
    int altKM = 0, neuKM;       // vorheriger/aktueller km-Stand
    int ltr;                    // getankte Liter

    // Aus Datei einlesen bis eof()
    do
    {
        std::getline(infile, data);
        if (!infile.eof())
        {
            // km-Stand aus String konvertieren
            std::from_chars_result res;
            if (res = std::from_chars(data.c_str(), data.c_str() + data.size(), neuKM); res.ec != std::errc())
            {
                std::cout << std::format("Fehler in km-Daten!\n{}\n", data);
            }
            else
            {
                // Liter aus String konvertieren
                if (res = std::from_chars(res.ptr + 1, data.c_str() + data.size(), ltr); res.ec != std::errc())
                {
                    std::cout << std::format("Fehler in Tankdaten!\n{}\n", data);
                }
                else
                {
                    // Falls vorheriger km-Stand vorliegt
                    if (altKM != 0)
                    {
                        // Verbrauchsdaten ausgeben
                        std::cout << std::format("km-Stand: {:4}, getankt: {:2} ltr, Verbrauch: {:.1f} ltr/100km\n",
                            neuKM, ltr, Verbrauch(neuKM - altKM, ltr));
                    }
                    // vorheriger km-Stand = aktueller km-Stand
                    altKM = neuKM;
                }

            }
        }
    } while (!infile.eof());
    // Datei wieder schliessen
    infile.close();

}
