#include <iostream>
#include <format>
#include <fstream>
#include <string>
#include <vector>
#include <tuple>
#include <algorithm>

constexpr const char* FILENAME = "../../04_Data/mitglieder.txt";

int main()
{
    // Tuple fuer Mitgliedsdaten: Name, Eintragsdatum (Tag, Monat, Jahr)
    using data = std::tuple<std::string, unsigned short, unsigned short, unsigned short>;
    // Vektor fuer Mitgliedsdaten
    std::vector<data> mitglieder;

    // Datei mit Mitgliederliste oeffnen
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        std::cout << std::format("Mitgliederliste {} nicht gefunden!\n", FILENAME);
        exit(1);
    }

    // Daten aus Datei einlesen
    do
    {
        unsigned short tag, monat;
        unsigned short jahr;
        std::string name;

        // Name einlesen
        std::getline(infile,name,',');
        if (!infile.eof())
        {
            // Komma ueberspringen
            infile.ignore(1);
            // Datum einlesen in getrennten Variablen
            // Punkt jeweils ueberspringen
            infile >> tag;
            infile.ignore(1);
            infile >> monat;
            infile.ignore(1);
            infile >> jahr;
            infile.ignore();
            // Daten in Vektor ablegen
            mitglieder.emplace_back(name,tag,monat,jahr);
        }

    } while (!infile.eof());
    // Datei schliessen
    infile.close();

    // Mitgliederliste ausgeben
    std::cout << "Mitgliederliste unsortiert:\n";
    for (auto [name, tag, monat, jahr] : mitglieder)
        std::cout << std::format("{}, {:02}.{:02}.{:04}\n",
            name, tag, monat, jahr);

    // Mitgliederliste nach Eintrittdatum sortieren
    std::ranges::sort(mitglieder,
              [] (const data& mitgl1, const data& mitgl2)
              {
                  auto [name1,tag1,monat1,jahr1] = mitgl1;
                  auto [name2,tag2,monat2,jahr2] = mitgl2;
                  if (jahr1 != jahr2)
                      return jahr1 < jahr2;
                  else
                      if (monat1 != monat2)
                          return monat1 < monat2;
                      else
                          return (tag1 < tag2);
              }
              );
    // Sortierte Mitgliederliste ausgeben
    std::cout << "\nMitgliederliste sortiert:\n";
    for (auto [name, tag, monat, jahr]: mitglieder)
        std::cout << std::format("{}, {:02}.{:02}.{:04}\n",
            name, tag, monat, jahr);
}
