
module;

#include <iostream>
#include <type_traits>
#include <cstring>

export module DynArray;

// Klasse implementiert einen gesicherten Zugriff auf
// ein Feld. Wird beim Schreiben über das Ende des Feldes
// hinausgegriffen, so wird das Feld entsprechend erweitert
// Standardmässig wird das Feld für die Ablage von int-
// Daten angelegt
// ACHTUNG!! Werden in dem Feld Objektzeiger abgelegt
// so müssen die dazugehörigen Objekte explizit wieder
// zerstört werden.
export template <typename T = int>
class DynArray
{
    T   *data; // Zeiger auf Datenfeld
    int size; // Grösse des Datenfelds
  public:
    // ctor, reserviert optional ein Feld mit einer
    // bestimmten Anzahl von Elementen
    DynArray(int reserveData = 0);
    // dtor
    ~DynArray();
    // Indexoperator für Array-Zugriff
    T& operator[] (int index);
    // Groesse des Feldes zurueckgeben
    int GetSize();
    // Ausgabe des DynArrays
    void PrintArray();
};
// ctor
// Standardmässig wird kein Feld allokiert und der Datenzeiger
// mit nullptr initialisiert
template <typename T>
DynArray<T>::DynArray(int reserveData): data(nullptr), size(0)
{
    // Falls Datenfeld angelegt werden soll
    if (reserveData)
    {
        // Datenfeld reservieren
        data = new T[reserveData];
        size = reserveData;
        // Wenn T ein intrinsischer Datentyp, Speicher mit 0 initialsieren
        if  (std::is_fundamental<T>::value == true)
            std::memset(data, 0, size * sizeof(T));
    }
}
// dtor, gibt reserviertes Feld frei
template <typename T>
DynArray<T>::~DynArray()
{
    delete [] data;
}
// Überladener Indexoperator für den Zugriff
// auf die Elemente im DynArray
template <typename T>
T& DynArray<T>::operator[] (int index)
{
    // Falls Index grösser als die Anzahl der Feldelement
    if (index>=size)
    {
        // Neues Feld mit der Grösse 'Index+1' reservieren
        // +1 hier unbedingt notwendig, da z.B. der
        // Index 10 das 11. Element adressiert!
        T *newData = new T[index+1];
        // Bisherige Daten in neues Feld übernehmen
        for (int copyOld = 0; copyOld < size; copyOld++)
        {
            newData[copyOld] = data[copyOld];
        }
        // Wenn T ein intrinsischer Datentyp, Speicher mit 0 initialsieren
        if (std::is_fundamental<T>::value)
            std::memset(&newData[size], 0, (index - size + 1) * sizeof(T));
        // Altes Feld nun entfernen
        delete [] data;
        // Und Zeiger auf neues Feld abspeichern
        data = newData;
        // Nicht vergessen: die Grösse korrigieren
        size = index+1;
    }
    // Gewünschtes Element zurückliefern
    return data[index];
}
template <typename T>
inline int DynArray<T>::GetSize()
{
    return size;
}
// Komplettes Feld ausgeben
template <typename T>
void DynArray<T>::PrintArray()
{
    for (int index=0; index<size; index++)
        std::cout << index << ':'  << data[index] << ", ";
    std::cout << '\n';
}
