module;
#include <iostream>
#include <fstream>
#include <sstream>
#include <typeinfo>
#include <string>

export module SaveFile;
using namespace std::string_literals;

// SaveFile implementiert eine Klasse zum sicheren Schreiben
// und Lesen von Daten aus eine Datei. Beim Einlesen der Daten
// wird der in der Datei mit abgelegte Datentyp mit dem einzulesenden
// Datentyp verglichen. Sind die Datentypen unterschiedlich,
// so wird eine Exception ausgelöst
export class SaveFile
{
    std::fstream ioFile;
public:
    // ctor, öffnet Datei zum Lesen und Schreiben
    SaveFile(const std::string& fileName);
    // dtor, schliesst Datei
    ~SaveFile();
    // Reset setzt den Schreib- und Lesezeiger zurück
    void Reset();
    // Member-Template zum Schreiben des Datum
    template <typename T> void Write(const T& value);
    // Member-Template zum Einlesen eines Datums
    template <typename T> void Read(T& value);
};
// ctor, öffnet Datei zum Lesen und Schreiben
SaveFile::SaveFile(const std::string& fileName)
{
    ioFile.open(fileName,std::ios::in|std::ios::out|std::ios::trunc);
    if (!ioFile)
       throw "Fehler beim Datei oeffnen!"s;
}
// dtor, schliesst Datei
SaveFile::~SaveFile()
{
    ioFile.close();
}
// Reset, setzt den Schreib- und Lesezeiger zurück
void SaveFile::Reset()
{
    ioFile.seekg(0);
    ioFile.seekp(0);
}
// Member-Template zum Schreiben des Datum
// Vor dem eigentlichen Datum wird der Datentyp mit
// in die Datei geschrieben. Trennzeichen zwischen
// Datentyp und Datum ist '@'
template <typename T>
void SaveFile::Write(const T& value)
{
    ioFile << typeid(T).name() << '@' << value << '\n';
}
// Member-Template zum Einlesen eines Datums
template <typename T>
void SaveFile::Read(T& value)
{
    // Datentyp und Datum einlesen
    std::string inputLine;
    std::getline(ioFile, inputLine);
    // Suche nach @ um Datentyp zu extrahieren
    auto delimiterPos = inputLine.find('@');
    if (delimiterPos == std::string::npos)
        throw "Datentyp konnte nicht bestimmt werden!"s;
    // Datentyp extrahieren
    std::string inputDType;
    inputDType.assign(inputLine,0,delimiterPos);
    // Falls einzulesender Datentyp und Dateidatentyp nicht
    // identisch sind, Exception auslösen
    if (inputDType != typeid(T).name())
    {
        std::string err = "Falscher Datentyp -> ist: " + inputDType + ", soll: " + typeid(T).name();
        throw err;
    }
    // Datentyp ueberspringen
    // ASCII-Datum nach binär konvertieren
    std::istringstream is(&inputLine[delimiterPos+1]);
    is >> value;
    if (is.fail())
       throw "Fehler bei der Datenkonvertierung!"s;
}
