// Modul zur Reservierung von Speicher mit
// new und malloc()
module;
#include <iostream>
#include <cstdlib>

export module Alloc;

// Templates zur Reservierung von Speicher.
// Templates werden nachher als Parameter eines weiteren
// Templates verwendet um die Art der Speicherreservierung
// zu bestimmen
// Hinweis: Die Templates sind jetzt vom Typ struct da
// alle Memberfunktionen public sind
// 1. Speicherreservierung mittels new
export template <typename T>
struct NewAlloc
{
    static T* Create()
    {
        std::cout << "NewAlloc::Create()\n";
        return new T;
    }
    static void Release(T *ptr)
    {
        std::cout << "NewAlloc::Release()\n";
        delete ptr;
    }
};
// 2. Speicherreservierung mittels malloc
// In der Praxis könnte z.B anstelle von malloc() die
// Reservierung von Speicher in einem bestimmten Bereich
// (z.B. shared-Memory) erfolgen
export template <typename T>
struct MallocAlloc
{
    static T* Create()
    {
        std::cout << "MallocAlloc::Create()\n";
        void *pMem = std::malloc(sizeof(T));
        // Wenn mit malloc Speicher fuer Objekte reserviert
        // wird, so muss der ctor des Objekts explizit mit
        // dem placement-new Operator aufgerufen werden
        if (pMem != NULL)
           return new(pMem) T;
        return NULL;
    }
    static void Release(T* ptr)
    {
        std::cout << "MallocAlloc::Release()\n";
        // Explizter Aufruf des dtors erforderlich!
        ptr->~T();
        free(ptr);
    }
};
