#include <iostream>
#include <format>
#include <fstream>
#include <vector>
#include <tuple>
#include <string>

// Datei mit Daten
constexpr const char* FNAME = "../../04_Data/aktien.csv";

int main()
{
    // tuple fuer Aktiendaten
    // Unternehmen, aktueller Kurs, hoechster Kurs, Datum Hoechststand
    using data = std::tuple<std::string, float,float,std::string>;
    // Vektor fuer Aktiendaten
    std::vector<data> aktien;

    // Datei oeffnen und Fehler abfangen
    std::ifstream infile(FNAME);
    if (!infile)
    {
        std::cout << std::format("Datei nicht gefunden: {}\n", FNAME);
        exit(1);
    }
    // Daten einlesen
    do
    {
        // Eingelesen Zeile
        std::string line;
        // Aktiendaten
        std::string unternehmen, datum;
        float aktKurs, hochKurs;
        // Zeile einlesen
        // Bei erreichen des Dateiendes den Rest überspringen
        if(getline(infile,unternehmen,';'); infile.eof())
            continue;
        // aktueller Kurs
        infile >> aktKurs;
        // Semikolon ueberspringen
        infile.ignore(1);
        // Hoechster Kurs
        infile >> hochKurs;
        // Semikolon ueberspringen
        infile.ignore(1);
        // Datum
        getline(infile,datum);
        // Daten in Vektor uebernehmen
        aktien.push_back({unternehmen, aktKurs, hochKurs, datum});

    } while (!infile.eof());
    // Datei wieder schliessen
    infile.close();

    int mindex = 0;         // Index des groessen Verlusts
    float mdiff = 1.0f;     // maximaler Verlust
    int index = 0;          // Indexzaehler
    // Vektor durchlaufen und Daten ablegen (structured binding!)
    for (const auto& [untern,akt,hoch,dat]: aktien)
    {
        // Daten ausgeben
        std::cout << std::format("{} akt: {:.2F}, max: {:.2F} am {}\n", untern, akt, hoch, dat);
        // Verlust in Prozent mit max. Verlust vergleichen
        if (akt/hoch < mdiff)
        {
            // Neuer max. Verlust gefunden und merken
            mdiff = akt/hoch;
            mindex = index;
        }
        // Index erhoehen
        index++;
    }
    // Daten des max. Verlust aus Vektor holen
    auto [untern,akt,hoch,dat] = aktien[mindex];
    // Und ausgeben
    std::cout << std::format("\nGroesster Verlust: {} {:.0f} %\n", untern, (1.0f - mdiff) * 100.);
}
