module;
#include <iostream>
#include <format>
#include <string>

export module Window;

// Definition der Basisklasse CWinBase
// CWinBase enthält die Grunddaten zur Beschreibung eines Fensters
// HINWEIS: Der protected-Konstruktor verhindert dass ein Objekt
// der Klasse CWinBase direkt erzeugt werden kann! Der Destruktor
// dagegen muss public sein
export class CWinBase
{
    short xPos, yPos; // Koordinaten des Fensters
    short width, height; // Grösse des Fensters
  protected: // protected Konstruktor!
    CWinBase(short xp, short yp, short wi, short he);
  public:
    virtual ~CWinBase() = default; // virtueller public-Destruktor
    virtual void Draw() const = 0; // pure virtual Memberfunktionen
};
// Definition der Memberfunktionen
// Konstruktor
CWinBase::CWinBase(short xp, short yp, short wi, short he)
{
    xPos = xp;
    yPos = yp;
    width = wi;
    height = he;
}
// Ausgabe der Position und Grösse
void CWinBase::Draw() const
{
    std::cout << std::format("Position: ({},{}), Groesse: ({},{})\n",
        xPos, yPos, width, height);
}

// Definition der Klasse CWindow
export class CWindow: public CWinBase
{
    std::string title; // Beschriftung
  public:
    CWindow(short xp, short yp, short wi, short he, const char* const t);
    virtual void Draw() const;
};
// Definition der Memberfunktionen
// Konstruktor
CWindow::CWindow(short xp, short yp, short wi, short he, const char* const t):
    CWinBase(xp,yp,wi,he),title(t)
{
}
// Ausgabe der Fensterdaten
void CWindow::Draw() const
{
    std::cout << std::format("Beschriftung: {}\n", title);
    CWinBase::Draw();
}
// Definition der Klasse CButton
export class CButton: public CWindow
{
    short buttonID; // Button-Nummer
  public:
    CButton(short xp, short yp, short wi, short he, const char* const t, short id);
    virtual void Draw() const;
};
// Definition der Memberfunktionen
// Konstruktor
CButton::CButton (short xp, short yp, short wi, short he, const char* const t, short id):
    CWindow(xp,yp,wi,he,t),
    buttonID(id)
{
}
// Ausgabe der Buttondaten
void CButton::Draw() const
{
    std::cout << std::format("Button-ID: {}\n", buttonID);
    // Achtung! Es wird Draw() von CWindow aufgerufen!
    CWindow::Draw();

}
